package ru.dbotthepony.kstarbound.client.render.entity

import ru.dbotthepony.kstarbound.PIXELS_IN_STARBOUND_UNITf
import ru.dbotthepony.kstarbound.client.ClientChunk
import ru.dbotthepony.kstarbound.client.gl.GLStateTracker
import ru.dbotthepony.kstarbound.client.gl.vertex.QuadTransformers
import ru.dbotthepony.kstarbound.client.gl.vertex.quadRotatedZ
import ru.dbotthepony.kstarbound.client.render.FrameAnimator
import ru.dbotthepony.kstarbound.client.render.SpriteAnimator
import ru.dbotthepony.kstarbound.client.render.makeSpriteAnimator
import ru.dbotthepony.kstarbound.world.entities.projectile.Projectile
import ru.dbotthepony.kvector.matrix.Matrix4fStack

open class ProjectileRenderer(state: GLStateTracker, entity: Projectile, chunk: ClientChunk?) : EntityRenderer(state, entity, chunk) {
	private val def = entity.def
	private val animator = def.image.makeSpriteAnimator(state, def.animationCycle, def.animationLoops)

	override fun render(stack: Matrix4fStack) {
		state.shaderVertexTexture.use()
		state.shaderVertexTexture.transform.set(stack.last)
		state.activeTexture = 0
		state.shaderVertexTexture["_texture"] = 0

		animator.advance()
		val sprite = animator.sprite
		sprite.texture.bind()

		val builder = state.flat2DTexturedQuads.small

		builder.begin()

		val width = (sprite.width / PIXELS_IN_STARBOUND_UNITf) / 2f
		val height = (sprite.height / PIXELS_IN_STARBOUND_UNITf) / 2f

		builder.quadRotatedZ(-width, -height, width, height, 5f, 0f, 0f, entity.movement.angle, sprite.transformer)

		builder.upload()
		builder.draw()
	}
}
