package ru.dbotthepony.kstarbound.defs.image

import com.google.gson.JsonSyntaxException
import com.google.gson.TypeAdapter
import com.google.gson.stream.JsonReader
import com.google.gson.stream.JsonToken
import com.google.gson.stream.JsonWriter
import ru.dbotthepony.kstarbound.Starbound

/**
 * Хранит данные (пару) вида "/example/animated.png" у которого, вероятнее всего, есть "/example/animated.frames"
 *
 * @see [AtlasConfiguration.Companion.get]
 */
data class ImageReference(
	val image: String,
	val config: AtlasConfiguration,
) {
	/**
	 * Вызывает [AtlasConfiguration.Companion.get] автоматически
	 *
	 * @see ImageReference
	 */
	constructor(image: String) : this(image, AtlasConfiguration.get(image))

	companion object : TypeAdapter<ImageReference>() {
		override fun write(out: JsonWriter, value: ImageReference) {
			out.value(value.image)
		}

		override fun read(`in`: JsonReader): ImageReference {
			if (`in`.peek() == JsonToken.STRING) {
				val image = Starbound.assetFolder(`in`.nextString())

				if (image.contains(':')) {
					throw JsonSyntaxException("Expected atlas/image reference, but got sprite reference: $image")
				}

				return ImageReference(image, AtlasConfiguration.get(image))
			}

			throw JsonSyntaxException("Expected atlas/image reference, but got: ${`in`.peek()}")
		}
	}
}
