package ru.dbotthepony.kstarbound.defs.item

import ru.dbotthepony.kstarbound.defs.enrollMap
import ru.dbotthepony.kstarbound.io.json.BuilderAdapter
import ru.dbotthepony.kstarbound.util.NotNullVar

class CurrencyItemPrototype : ItemPrototype(), ICurrencyItemDefinition {
	override var pickupSoundsSmall: List<String> = listOf()
	override var pickupSoundsMedium: List<String> = listOf()
	override var pickupSoundsLarge: List<String> = listOf()
	override var smallStackLimit: Long by NotNullVar()
	override var mediumStackLimit: Long by NotNullVar()
	override var currency: String by NotNullVar()
	override var value: Long by NotNullVar()

	init {
		maxStack = 16777216L
	}

	override fun assemble(): IItemDefinition {
		return CurrencyItemDefinition(
			shortdescription = shortdescription,
			description = description,
			itemName = itemName,
			price = price,
			rarity = rarity,
			category = category,
			inventoryIcon = inventoryIcon,
			itemTags = itemTags,
			learnBlueprintsOnPickup = learnBlueprintsOnPickup,
			maxStack = maxStack,
			eventCategory = eventCategory,
			consumeOnPickup = consumeOnPickup,
			pickupQuestTemplates = pickupQuestTemplates,
			scripts = scripts,
			tooltipKind = tooltipKind,
			twoHanded = twoHanded,
			radioMessagesOnPickup = radioMessagesOnPickup,
			fuelAmount = fuelAmount,

			json = enrollMap(json),

			pickupSoundsSmall = pickupSoundsSmall,
			pickupSoundsMedium = pickupSoundsMedium,
			pickupSoundsLarge = pickupSoundsLarge,
			smallStackLimit = smallStackLimit,
			mediumStackLimit = mediumStackLimit,
			currency = currency,
			value = value,
		)
	}

	companion object {
		val ADAPTER = BuilderAdapter.Builder(::CurrencyItemPrototype)
			.also { addFields(it as BuilderAdapter.Builder<ItemPrototype>) } // безопасность: свойства родительского класса объявлены как final
			.autoList(CurrencyItemPrototype::pickupSoundsSmall)
			.autoList(CurrencyItemPrototype::pickupSoundsMedium)
			.autoList(CurrencyItemPrototype::pickupSoundsLarge)
			.auto(CurrencyItemPrototype::smallStackLimit)
			.auto(CurrencyItemPrototype::mediumStackLimit)
			.auto(CurrencyItemPrototype::currency)
			.auto(CurrencyItemPrototype::value)
			.build()
	}
}
