package ru.dbotthepony.kstarbound.defs.item

import ru.dbotthepony.kstarbound.defs.MaterialReference
import ru.dbotthepony.kstarbound.defs.enrollMap
import ru.dbotthepony.kstarbound.io.json.BuilderAdapter

class LiquidItemPrototype : ItemPrototype() {
	var liquid: MaterialReference? = null

	var liquidId: Int?
		get() = liquid?.id
		set(value) {
			if (liquid == null)
				liquid = MaterialReference(id = value, name = null)
		}

	var liquidName: String?
		get() = liquid?.name
		set(value) { liquid = MaterialReference(name = value, id = null) }

	override fun assemble(): IItemDefinition {
		return LiquidItemDefinition(
			shortdescription = shortdescription,
			description = description,
			itemName = itemName,
			price = price,
			rarity = rarity,
			category = category,
			inventoryIcon = inventoryIcon,
			itemTags = itemTags,
			learnBlueprintsOnPickup = learnBlueprintsOnPickup,
			maxStack = maxStack,
			eventCategory = eventCategory,
			consumeOnPickup = consumeOnPickup,
			pickupQuestTemplates = pickupQuestTemplates,
			scripts = scripts,
			tooltipKind = tooltipKind,
			twoHanded = twoHanded,
			radioMessagesOnPickup = radioMessagesOnPickup,
			fuelAmount = fuelAmount,

			liquid = checkNotNull(liquid) { "Liquid is null (either 'liquidId' or 'liquidName' should be present, or 'liquid' itself)" },

			json = enrollMap(json),
		)
	}

	companion object {
		val ADAPTER = BuilderAdapter.Builder(::LiquidItemPrototype)
			.also { addFields(it as BuilderAdapter.Builder<ItemPrototype>) } // безопасность: свойства родительского класса объявлены как final
			.auto(LiquidItemPrototype::liquid)
			.auto(LiquidItemPrototype::liquidId)
			.auto(LiquidItemPrototype::liquidName)
			.build()
	}
}
