package ru.dbotthepony.kstarbound.defs.item

import ru.dbotthepony.kstarbound.defs.MaterialReference
import ru.dbotthepony.kstarbound.defs.enrollMap
import ru.dbotthepony.kstarbound.io.json.BuilderAdapter

class MaterialItemPrototype : ItemPrototype() {
	var material: MaterialReference? = null

	var materialId: Int?
		get() = material?.id
		set(value) {
			if (material == null)
				material = MaterialReference(id = value, name = null)
		}

	var materialName: String?
		get() = material?.name
		set(value) { material = MaterialReference(name = value, id = null) }

	override fun assemble(): IItemDefinition {
		return MaterialItemDefinition(
			shortdescription = shortdescription,
			description = description,
			itemName = itemName,
			price = price,
			rarity = rarity,
			category = category,
			inventoryIcon = inventoryIcon,
			itemTags = itemTags,
			learnBlueprintsOnPickup = learnBlueprintsOnPickup,
			maxStack = maxStack,
			eventCategory = eventCategory,
			consumeOnPickup = consumeOnPickup,
			pickupQuestTemplates = pickupQuestTemplates,
			scripts = scripts,
			tooltipKind = tooltipKind,
			twoHanded = twoHanded,
			radioMessagesOnPickup = radioMessagesOnPickup,
			fuelAmount = fuelAmount,

			material = checkNotNull(material) { "Material is null (either 'materialId' or 'materialName' should be present, or 'material' itself)" },

			json = enrollMap(json),
		)
	}

	companion object {
		val ADAPTER = BuilderAdapter.Builder(::MaterialItemPrototype)
			.also { addFields(it as BuilderAdapter.Builder<ItemPrototype>) } // безопасность: свойства родительского класса объявлены как final
			.auto(MaterialItemPrototype::material)
			.auto(MaterialItemPrototype::materialId)
			.auto(MaterialItemPrototype::materialName)
			.build()
	}
}
