package ru.dbotthepony.kstarbound.io.json

import com.google.gson.JsonSyntaxException
import com.google.gson.TypeAdapter
import com.google.gson.stream.JsonReader
import com.google.gson.stream.JsonToken
import com.google.gson.stream.JsonWriter

fun <T> TypeAdapter<T>.transformRead(transformer: (T) -> T): TypeAdapter<T> {
	return object : TypeAdapter<T>() {
		override fun write(out: JsonWriter, value: T) {
			return this@transformRead.write(out, value)
		}

		override fun read(`in`: JsonReader): T {
			return transformer(this@transformRead.read(`in`))
		}
	}
}

fun <T> TypeAdapter<T>.transformWrite(transformer: (T) -> T): TypeAdapter<T> {
	return object : TypeAdapter<T>() {
		override fun write(out: JsonWriter, value: T) {
			return this@transformWrite.write(out, transformer(value))
		}

		override fun read(`in`: JsonReader): T {
			return this@transformWrite.read(`in`)
		}
	}
}

fun <In, Out> TypeAdapter<In>.transform(read: (In) -> Out, write: (Out) -> In): TypeAdapter<Out> {
	return object : TypeAdapter<Out>() {
		override fun write(out: JsonWriter, value: Out) {
			return this@transform.write(out, write(value))
		}

		override fun read(`in`: JsonReader): Out {
			return read(this@transform.read(`in`))
		}
	}
}

fun <T> TypeAdapter<T>.ifString(reader: (String) -> T): TypeAdapter<T> {
	return object : TypeAdapter<T>() {
		override fun write(out: JsonWriter, value: T) {
			return this@ifString.write(out, value)
		}

		override fun read(`in`: JsonReader): T {
			if (`in`.peek() == JsonToken.STRING) {
				return reader(`in`.nextString())
			}

			return this@ifString.read(`in`)
		}
	}
}

fun <T> TypeAdapter<T?>.neverNull(): TypeAdapter<T> {
	return object : TypeAdapter<T>() {
		override fun write(out: JsonWriter, value: T) {
			this@neverNull.write(out, value)
		}

		override fun read(`in`: JsonReader): T {
			val path = `in`.path
			return this@neverNull.read(`in`) ?: throw JsonSyntaxException("Value was null near $path")
		}
	}
}

fun <T> TypeAdapter<T>.allowNull(): TypeAdapter<T?> = nullSafe()
